(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      5722,        177]*)
(*NotebookOutlinePosition[      6401,        200]*)
(*  CellTagsIndexPosition[      6357,        196]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["The IIR Filter Architecture Package", "Title",
  InitializationCell->True],

Cell["Clustered Look-Ahead with incremental output computation", "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(BeginPackage["\<Architecture`CLAIncr`\>", "\<Architecture`Main`\>"]\)], \
"Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`CLAIncr`"\)], "Output",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Begin["\<`Private`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`CLAIncr`Private`"\)], "Output",
  InitializationCell->True]
}, Open  ]],

Cell[BoxData[
    \(Description["\<CLAIncr\>"] := "\<The CLAIncr architecture is based on \
the clustered look-ahead approach with incremental output computation. Also \
called Direct Form Block Filter by Parhi. Fast FIR filter techniques might be \
used to improve this architecture further. Note that this filter is not \
necessarily stable. See: parhi89pipeline2.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ParamNr["\<CLAIncr\>"] := 3\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamDescription["\<CLAIncr\>"]\)[
        3] := "\<The number of pipeline stages  in the recursive loop of the \
implementation. Must be a power of two. Default = 1.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamTest["\<CLAIncr\>"]\)[param_, 3] := 
      And[param[\([3]\)] \[GreaterEqual] \ 1, 
        Round[Log[2, param[\([3]\)]]] \[Equal] \ 
          Log[2, param[\([3]\)]]]\)], "Input",
  InitializationCell->True],

Cell["\<\
First find the necessary pipelining level to make the response time one, and \
then turn towards calculating the number of simultaneous inputs.\
\>", "Text",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamFindThroughput["\<CLAIncr\>"]\)[n_, t_] := 
      Module[{p, m, sol}, \[IndentingNewLine]m = 
          1; \[IndentingNewLine]While[\(ResponseTime["\<CLAIncr\>"]\)[{n, p, 
                m}] > 1, m = 2  m]; \[IndentingNewLine]sol\  = \ 
          Solve[\(Throughput["\<CLAIncr\>"]\)[{n, p, m}]\  \[Equal] \ t, p]; 
        If[Length[sol] \[Equal] 0, \ {0, 0, 0}, {n, p, m}\  /. \ 
            sol[\([1]\)]]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(WorkUnit["\<CLAIncr\>"]\)[{N_, P_, M_:  1}] := {1, N}/M\)], "Input",
  InitializationCell->True],

Cell["\<\
Note that the number of multipliers described in parhi89pipeline2 is wrong. \
The author misplaced a closing brace. The correct formula (as far as I know) \
is below.\
\>", "Text",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareMultipliers["\<CLAIncr\>"]\)[{N_, P_, M_:  1}] := 
      If[P \[GreaterEqual] \ N, 
        P \((3\ N\  + \ 1)\)\  - \ \((\((N + 1)\) N/2)\) + 
          N\^2\ Log[2, M], \((2\ N\  + \ \((P + 1)\)/2)\) P\  + \ 
          N\ P\ Log[2, M]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareAdders["\<CLAIncr\>"]\)[{N_, P_, M_:  1}] := 
      If[N \[GreaterEqual] \ P, 
        P \((3\ N\  + \ 1)\)\  - \ \((\((N + 1)\) N/2)\) + \ 
          N\^2\ Log[2, M], \((2\ N\  + \ \((P + 1)\)/2)\) P + \ 
          N\ P\ Log[2, M]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareBuffers["\<CLAIncr\>"]\)[{N_, P_, M_:  1}] := \ 
      M\ N\  + \ N\  + \ \((M\  - \ 1)\)\ P\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Latency["\<CLAIncr\>"]\)[{N_, P_, M_:  1}] := \(ResponseTime[arch_]\)[
          param_]*\((\((2  N\  + \ M + \ P\ )\)/N\  + \ 
            Log[2, M])\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(End[]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`CLAIncr`Private`"\)], "Output",
  InitializationCell->True]
}, Open  ]],

Cell[BoxData[
    \(EndPackage[]\)], "Input",
  InitializationCell->True]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1400}, {0, 975}},
AutoGeneratedPackage->Automatic,
WindowSize->{496, 740},
WindowMargins->{{Automatic, 238}, {Automatic, 24}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 80, 1, 150, "Title",
  InitializationCell->True],
Cell[1859, 56, 104, 1, 80, "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1988, 61, 130, 3, 50, "Input",
  InitializationCell->True],
Cell[2121, 66, 85, 2, 29, "Output",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[2243, 73, 83, 2, 30, "Input",
  InitializationCell->True],
Cell[2329, 77, 93, 2, 29, "Output",
  InitializationCell->True]
}, Open  ]],
Cell[2437, 82, 410, 6, 170, "Input",
  InitializationCell->True],
Cell[2850, 90, 88, 2, 30, "Input",
  InitializationCell->True],
Cell[2941, 94, 232, 4, 90, "Input",
  InitializationCell->True],
Cell[3176, 100, 235, 5, 70, "Input",
  InitializationCell->True],
Cell[3414, 107, 195, 4, 52, "Text",
  InitializationCell->True],
Cell[3612, 113, 476, 8, 150, "Input",
  InitializationCell->True],
Cell[4091, 123, 118, 2, 30, "Input",
  InitializationCell->True],
Cell[4212, 127, 218, 5, 52, "Text",
  InitializationCell->True],
Cell[4433, 134, 308, 6, 72, "Input",
  InitializationCell->True],
Cell[4744, 142, 303, 6, 72, "Input",
  InitializationCell->True],
Cell[5050, 150, 163, 3, 50, "Input",
  InitializationCell->True],
Cell[5216, 155, 215, 4, 70, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[5456, 163, 66, 2, 30, "Input",
  InitializationCell->True],
Cell[5525, 167, 93, 2, 29, "Output",
  InitializationCell->True]
}, Open  ]],
Cell[5633, 172, 73, 2, 30, "Input",
  InitializationCell->True]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

