(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      7649,        214]*)
(*NotebookOutlinePosition[      8328,        237]*)
(*  CellTagsIndexPosition[      8284,        233]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["The IIR Filter Architecture Package", "Title",
  InitializationCell->True],

Cell["Incremental Block-State", "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(BeginPackage["\<Architecture`IncrementalBlockState`\>", \
"\<Architecture`Main`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`IncrementalBlockState`"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Begin["\<`Private`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`IncrementalBlockState`Private`"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Description["\<IncrementalBlockState\>"] := "\<The incremental \
block-state architecture, like the block-state architecture, is based on the \
state space form of the filter. For the performance calculations we assumed \
that the decomposition of the filter into parallel second order sections is \
known and that there are no first order sections. The third parameter is the \
increment size, but it can be omitted. In this case the third parameter will \
be chosen such that the number of multipliers in the implementation is \
minimal.See: parhi87block, parhi89pipeline2.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ParamNr["\<IncrementalBlockState\>"] := 4\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamDescription["\<IncrementalBlockState\>"]\)[
        3] := "\<The increment size. Outputs will be calculated in groups of \
this size. This means that the increment size should be between 1 and the \
number of simultaneous inputs/outputs. Note that if the increment size is \
half of that or larger, then this architecture is the same as the BlockState \
architecture.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamDescription["\<IncrementalBlockState\>"]\)[
        4] := "\<The number of pipeline stages  in the recursive loop of the \
implementation. Must be a power of two. Default = 1.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamMax["\<IncrementalBlockState\>"]\)[param_, 3] := 
      param[\([2]\)]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamTest["\<IncrementalBlockState\>"]\)[param_, 4] := 
      And[param[\([4]\)] \[GreaterEqual] \ 1, 
        Round[Log[2, param[\([4]\)]]] \[Equal] \ 
          Log[2, param[\([4]\)]]]\)], "Input",
  InitializationCell->True],

Cell["\<\
First find the necessary pipelining level to make the response time one, and \
then turn towards calculating the number of simultaneous inputs.\
\>", "Text",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamFindThroughput["\<IncrementalBlockState\>"]\)[n_, t_] := 
      Block[{p, i, m, sol}, \ \[IndentingNewLine]m = 
          1; \[IndentingNewLine]While[\(ResponseTime["\<IncrementalBlockState\
\>"]\)[{n, p, i, m}] > 1, m = 2  m]; \[IndentingNewLine]sol\  = \ 
          Solve[\(Throughput["\<IncrementalBlockState\>"]\)[{n, p, i, 
                  m}]\  \[Equal] \ t, p]; \[IndentingNewLine]p = 
          p\  /. \ sol[\([1]\)]; \[IndentingNewLine]i = 
          FindIncrement[n, p, m]; \[IndentingNewLine]{n, p, i, m}]\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(WorkUnit["\<IncrementalBlockState\>"]\)[{N_, P_, i_:  1, 
          M_:  1}] := {1, 2}/M\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareMultipliers["\<IncrementalBlockState\>"]\)[{n_, P_, I_, 
          M_:  1}] := 
      2  n \((P + Quotient[P, I])\)\  + \ \((P \((I + 1)\))\)/
          2\  + \ \((Mod[P, I] \((I + Mod[P, I])\))\)/2\  + \ 
        n\ I \((Quotient[P, I] - 1)\)\  + \ 2\ n\ Log[2, M]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareAdders["\<IncrementalBlockState\>"]\)[{n_, P_, I_, M_:  1}] := 
      2\ n\ \((P + Quotient[P, I])\)\  + \ \((P \((I + 1)\))\)/
          2\  + \ \((Mod[P, I] \((I + Mod[P, I])\))\)/2\  + \ 
        n\ I \((Quotient[P, I] - 1)\) + \ 2\ n\ Log[2, M]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareBuffers["\<IncrementalBlockState\>"]\)[{N_, P_, i_:  1, 
          M_:  1}] := \ N\ M\  + \ \((M - 1)\) \((P + N)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Latency["\<IncrementalBlockState\>"]\)[{N_, P_, I_:  1, 
          M_:  1}] := \(ResponseTime[arch_]\)[{N, P, I, 
            M}]*\((N\  + \ 2\ P\  + Quotient[P, I] - \ 2)\)/2\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(FindIncrement[N_, P_, M_:  1] := 
      Block[{i, opt, opti}, opt\  = \ Infinity; 
        For[i\  = \ 1, \ i \[LessEqual] \((P/2)\)\  + \ 1, 
          If[\(HardwareMultipliers["\<IncrementalBlockState\>"]\)[{N, P, i, 
                  M}] < opt, 
            opt = \(HardwareMultipliers["\<IncrementalBlockState\>"]\)[{N, P, 
                  i, M}]; opti = i; \(i++\), \(i++\)]]; opti]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareMultipliers["\<IncrementalBlockState\>"]\)[{N_, 
          P_}] := \(HardwareMultipliers["\<IncrementalBlockState\>"]\)[{N, 
          P, \ FindIncrement[N, P]}]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareAdders["\<IncrementalBlockState\>"]\)[{N_, 
          P_}] := \(HardwareAdders["\<IncrementalBlockState\>"]\)[{N, P, 
          FindIncrement[N, P]}]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Latency["\<IncrementalBlockState\>"]\)[{N_, 
          P_}] := \(Latency["\<IncrementalBlockState\>"]\)[{N, P, 
          FindIncrement[N, P]}]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(End[]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`IncrementalBlockState`Private`"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(EndPackage[]\)], "Input",
  InitializationCell->True]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1400}, {0, 975}},
AutoGeneratedPackage->Automatic,
WindowSize->{496, 740},
WindowMargins->{{123, Automatic}, {Automatic, 34}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 80, 1, 150, "Title",
  InitializationCell->True],
Cell[1859, 56, 71, 1, 51, "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1955, 61, 144, 3, 50, "Input",
  InitializationCell->True],
Cell[2102, 66, 71, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2210, 72, 83, 2, 30, "Input",
  InitializationCell->True],
Cell[2296, 76, 79, 1, 29, "Output"]
}, Open  ]],
Cell[2390, 80, 639, 9, 270, "Input",
  InitializationCell->True],
Cell[3032, 91, 102, 2, 30, "Input",
  InitializationCell->True],
Cell[3137, 95, 438, 7, 170, "Input",
  InitializationCell->True],
Cell[3578, 104, 246, 4, 90, "Input",
  InitializationCell->True],
Cell[3827, 110, 138, 3, 50, "Input",
  InitializationCell->True],
Cell[3968, 115, 249, 5, 70, "Input",
  InitializationCell->True],
Cell[4220, 122, 195, 4, 52, "Text",
  InitializationCell->True],
Cell[4418, 128, 588, 10, 250, "Input",
  InitializationCell->True],
Cell[5009, 140, 151, 3, 50, "Input",
  InitializationCell->True],
Cell[5163, 145, 336, 6, 110, "Input",
  InitializationCell->True],
Cell[5502, 153, 319, 5, 110, "Input",
  InitializationCell->True],
Cell[5824, 160, 186, 3, 50, "Input",
  InitializationCell->True],
Cell[6013, 165, 238, 4, 90, "Input",
  InitializationCell->True],
Cell[6254, 171, 451, 8, 150, "Input",
  InitializationCell->True],
Cell[6708, 181, 232, 4, 90, "Input",
  InitializationCell->True],
Cell[6943, 187, 220, 4, 70, "Input",
  InitializationCell->True],
Cell[7166, 193, 206, 4, 70, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[7397, 201, 66, 2, 30, "Input",
  InitializationCell->True],
Cell[7466, 205, 79, 1, 29, "Output"]
}, Open  ]],
Cell[7560, 209, 73, 2, 30, "Input",
  InitializationCell->True]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

