(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      7897,        220]*)
(*NotebookOutlinePosition[      8576,        243]*)
(*  CellTagsIndexPosition[      8532,        239]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["The IIR Filter Architecture Package", "Title",
  InitializationCell->True],

Cell["Incremental Block-State All-pass", "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(BeginPackage["\<Architecture`IncrementalBlockStateAllpass`\>", \
"\<Architecture`Main`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`IncrementalBlockStateAllpass`"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Begin["\<`Private`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`IncrementalBlockStateAllpass`Private`"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Description["\<IncrementalBlockStateAllpass\>"] := "\<The incremental \
block-state all-pass architecture, is based on the block-state all-pass \
architecture like the incremental block-state architecture is based on the \
block-state architecture. Again the drawback is that the A matrix becomes \
full.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ParamNr["\<IncrementalBlockStateAllpass\>"] := 4\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamDescription["\<IncrementalBlockStateAllpass\>"]\)[
        3] := "\<The increment size. Outputs will be calculated in groups of \
this size. This means that the increment size should be between 1 and the \
number of simultaneous inputs/outputs. Note that if the increment size is \
half of that or larger, then this architecture is the same as the \
BlockStateAllpass architecture.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamDescription["\<IncrementalBlockStateAllpass\>"]\)[
        4] := "\<The number of pipeline stages  in the recursive loop of the \
implementation. Must be a power of two. Default = 1.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamMax["\<IncrementalBlockStateAllpass\>"]\)[param_, 3] := 
      param[\([2]\)]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamTest["\<IncrementalBlockStateAllpass\>"]\)[param_, 4] := 
      And[param[\([4]\)] \[GreaterEqual] \ 1, 
        Round[Log[2, param[\([4]\)]]] \[Equal] \ 
          Log[2, param[\([4]\)]]]\)], "Input",
  InitializationCell->True],

Cell["\<\
First find the necessary pipelining level to make the response time one, and \
then turn towards calculating the number of simultaneous inputs.\
\>", "Text",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamFindThroughput["\<IncrementalBlockStateAllpass\>"]\)[n_, t_] := 
      Block[{p, i, m, sol}, \ \[IndentingNewLine]m = 
          1; \[IndentingNewLine]While[\(ResponseTime["\<\
IncrementalBlockStateAllpass\>"]\)[{n, p, i, m}] > 1, 
          m = 2  m]; \[IndentingNewLine]sol\  = \ 
          Solve[\(Throughput["\<IncrementalBlockStateAllpass\>"]\)[{n, p, i, 
                  m}]\  \[Equal] \ t, p]; \[IndentingNewLine]p = 
          p\  /. \ sol[\([1]\)]; \[IndentingNewLine]i = 
          FindIncrement[n, p, m]; \[IndentingNewLine]{n, p, i, m}]\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(WorkUnit["\<IncrementalBlockStateAllpass\>"]\)[{N_, P_, i_:  1, 
          M_:  1}] := {1, N}/M\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareMultipliers["\<IncrementalBlockStateAllpass\>"]\)[{n_, P_, 
          I_, M_:  1}] := 
      2  n \((P + Quotient[P, I])\)\  + \ \((P \((I + 1)\))\)/
          2\  + \ \((Mod[P, I] \((I + Mod[P, I])\))\)/2\  + \ 
        n\ I \((Quotient[P, I] - 1)\)\  + \ n\^2\ Log[2, M]\  - 2  n\  + \ 
        n\^2 - \((\((I + Mod[P, I])\)\ n\  - \ 
            n \((n + 1)\)/2)\)\  - \((Quotient[P, I] - 1)\) \((I\ n\  - \ 
              n \((n + 1)\)/2)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareAdders["\<IncrementalBlockStateAllpass\>"]\)[{n_, P_, I_, 
          M_:  1}] := 
      2\ n\ \((P + Quotient[P, I])\)\  + \ \((P \((I + 1)\))\)/
          2\  + \ \((Mod[P, I] \((I + Mod[P, I])\))\)/2\  + \ 
        n\ I \((Quotient[P, I] - 1)\) + \ n\^2\ Log[2, M] - 2  n\  + \ 
        n\^2\  - \((\((I + Mod[P, I])\)\ n\  - \ 
            n \((n + 1)\)/2)\)\  - \((Quotient[P, I] - 1)\) \((I\ n\  - \ 
              n \((n + 1)\)/2)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareBuffers["\<IncrementalBlockStateAllpass\>"]\)[{N_, P_, 
          i_:  1, M_:  1}] := \ N\ M\  + \ \((M - 1)\) \((P + N)\)\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(Latency["\<IncrementalBlockStateAllpass\>"]\)[{N_, P_, I_:  1, 
          M_:  1}] := \(ResponseTime[arch_]\)[{N, P, I, 
            M}]*\((N\  + \ 2\ P\  + Quotient[P, I] - \ 2)\)/N\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(FindIncrement[N_, P_, M_:  1] := 
      Block[{i, opt, opti}, opt\  = \ Infinity; 
        For[i\  = \ 1, \ i \[LessEqual] \((P/2)\)\  + \ 1, 
          If[\(HardwareMultipliers["\<IncrementalBlockStateAllpass\>"]\)[{N, 
                  P, i, M}] < opt, 
            opt = \(HardwareMultipliers["\<IncrementalBlockStateAllpass\>"]\)[\
{N, P, i, M}]; opti = i; \(i++\), \(i++\)]]; opti]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareMultipliers["\<IncrementalBlockStateAllpass\>"]\)[{N_, 
          P_}] := \(HardwareMultipliers["\<IncrementalBlockStateAllpass\>"]\)[\
{N, P, \ FindIncrement[N, P]}]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareAdders["\<IncrementalBlockState\>"]\)[{N_, 
          P_}] := \(HardwareAdders["\<IncrementalBlockStateAllpass\>"]\)[{N, 
          P, FindIncrement[N, P]}]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Latency["\<IncrementalBlockStateAllpass\>"]\)[{N_, 
          P_}] := \(Latency["\<IncrementalBlockState\>"]\)[{N, P, 
          FindIncrement[N, P]}]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(End[]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`IncrementalBlockStateAllpass`Private`"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(EndPackage[]\)], "Input",
  InitializationCell->True]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1400}, {0, 975}},
AutoGeneratedPackage->Automatic,
WindowSize->{496, 740},
WindowMargins->{{235, Automatic}, {Automatic, 12}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 80, 1, 150, "Title",
  InitializationCell->True],
Cell[1859, 56, 80, 1, 51, "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1964, 61, 151, 3, 70, "Input",
  InitializationCell->True],
Cell[2118, 66, 78, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2233, 72, 83, 2, 30, "Input",
  InitializationCell->True],
Cell[2319, 76, 86, 1, 29, "Output"]
}, Open  ]],
Cell[2420, 80, 368, 6, 150, "Input",
  InitializationCell->True],
Cell[2791, 88, 109, 2, 30, "Input",
  InitializationCell->True],
Cell[2903, 92, 452, 7, 170, "Input",
  InitializationCell->True],
Cell[3358, 101, 253, 4, 90, "Input",
  InitializationCell->True],
Cell[3614, 107, 145, 3, 50, "Input",
  InitializationCell->True],
Cell[3762, 112, 256, 5, 70, "Input",
  InitializationCell->True],
Cell[4021, 119, 195, 4, 52, "Text",
  InitializationCell->True],
Cell[4219, 125, 620, 11, 250, "Input",
  InitializationCell->True],
Cell[4842, 138, 158, 3, 50, "Input",
  InitializationCell->True],
Cell[5003, 143, 515, 9, 150, "Input",
  InitializationCell->True],
Cell[5521, 154, 509, 9, 130, "Input",
  InitializationCell->True],
Cell[6033, 165, 195, 4, 50, "Input",
  InitializationCell->True],
Cell[6231, 171, 245, 4, 90, "Input",
  InitializationCell->True],
Cell[6479, 177, 448, 8, 230, "Input",
  InitializationCell->True],
Cell[6930, 187, 237, 4, 90, "Input",
  InitializationCell->True],
Cell[7170, 193, 227, 4, 70, "Input",
  InitializationCell->True],
Cell[7400, 199, 213, 4, 70, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[7638, 207, 66, 2, 30, "Input",
  InitializationCell->True],
Cell[7707, 211, 86, 1, 29, "Output"]
}, Open  ]],
Cell[7808, 215, 73, 2, 30, "Input",
  InitializationCell->True]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

