(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     13965,        410]*)
(*NotebookOutlinePosition[     14643,        433]*)
(*  CellTagsIndexPosition[     14599,        429]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["The IIR Filter Architecture Package", "Title",
  InitializationCell->True],

Cell["Main File", "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(BeginPackage["\<Architecture`Main`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`Main`"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(ArchitectureListIIR := {"\<BlockState\>", \
"\<IncrementalBlockState\>", "\<ParallelBlockState\>", "\<SLA\>", \
"\<SLAIncr\>", "\<CLAIncr\>"};\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ArchitectureListAllpass := {"\<CLAAllpass\>", \
"\<BlockStateAllpass\>", \ "\<IncrementalBlockStateAllpass\>"};\)\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(ArchitectureList := 
        Union[ArchitectureListIIR, ArchitectureListAllpass];\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell["Usage Descriptions", "Section",
  InitializationCell->True],

Cell[BoxData[
    \(\(ArchitectureList::usage = "\<ArchitectureList gives a list of valid \
architecture handles.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ArchitectureListIIR::usage = "\<ArchitectureListIIR gives a list of \
valid architecture handles for general IIR architectures.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ArchitectureListAllpass::usage = "\<ArchitectureListAllpass gives a \
list of valid architecture handles for architectures specifically designed \
for Allpass filters.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Description::usage = "\<Description[arch] gives a description of the \
architecture, and references to the literature.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamNr::usage = "\<ParamNr[arch] gives the number of parameters \
needed by the architecture. This is at least two for any \
architecture\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamDescription::usage = \ "\<ParamDescription[arch][n] gives a \
description of the n-th parameter of the architecture. 1 \[LessEqual] n \
\[LessEqual] ParamNr[arch]\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamMin::usage = "\<ParamMin[arch][param, n] gives the minimum \
possible value of the n-th parameter of the architecture. 1 \[LessEqual] n \
\[LessEqual] ParamNr[arch]\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamMax::usage = "\<ParamMin[arch][param, n] gives the maximum \
possible value of the n-th parameter of the architecture. 1 \[LessEqual] n \
\[LessEqual] ParamNr[arch]\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ParamFindHardwareMultipliers::usage = \
"\<ParamFindHardwareMultipliers[arch][n,t] gives the parameters for an \
instance of the architecture, such that it contains t hardware multipliers. \
If this is not possible, it wil return a list containing only zeros. The \
parameters returned by this function do not have to be valid, use the \
ParamTest function to check this.\>"\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(ParamFindThroughput::usage = "\<ParamFindThroughput[arch][n,t] gives \
the parameters for an instance of the architecture, such that it can handle a \
filter of order n at a throughput of t. If this is not possible, it wil \
return a list containing only zeros. The parameters returned by this function \
do not have to be valid, use the ParamTest function to check this.\>"\)], \
"Input",
  InitializationCell->True],

Cell[BoxData[
    \("ParamFindThroughput[arch][n,t] gives the parameters for an instance of \
the architecture, such that it can handle a filter of order n at a throughput \
of t. If this is not possible, it wil return a list containing only zeros. \
The parameters returned by this function do not have to be valid, use the \
ParamTest function to check this."\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ParamTest::usage = "\<ParamTest[arch][param,n] tests whether the value \
of the n-th parameter is valid for the architecture (and with respect to the \
other parameters).
ParamTest[arch][param] tests whether the all parameters are valid for the \
architecture.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("ParamTest[arch][param,n] tests whether the value of the n-th parameter \
is valid for the architecture (and with respect to the other parameters). \
ParamTest[arch][param] tests whether the all parameters are valid for the \
architecture."\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(ClockCycle::usage = \*"\"\<ClockCycle[arch][param] gives the length \
of a clock cycle of the architecture. This length is expressed in \!\(T\_m\) \
and \!\(T\_a\), representing the time it takes to do a multiplication and \
addition respectively. See also WorkUnit and NeededCycles.\>\"";\)\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(NeededCycles::usage = "\<NeededCycles[arch][param, {mults, adds}] \
gives the number of clock cycles required to perform mults multiplications \
and adds additions. This function returns 1 by default, but can be overridden \
to simulate hardware restrictions. See also ClockCycle and \
WorkUnit.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(WorkUnit::usage = "\<WorkUnit[arch][param] gives a list of two \
elements containing the number of multiplications and additions respectively \
needed (ideally) in a single cycle. Note the word 'ideally', whether or not \
this is possible depends on the NeededCycles function. See also ClockCycle \
and NeededCycles.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareMultipliers::usage = "\<HardwareMultipliers[arch][param] \
gives the number of multipliers needed by the architecture.\>";\)\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareAdders::usage = "\<HardwareAdders[arch][param] gives the \
number of adders needed by the architecture.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareBuffers::usage = "\<HardwareBuffers[arch][param] gives the \
number of buffers needed by the architecture.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Latency::usage\  = \ "\<Latency[arch][param] (average) latency of the \
architecture in clock cycles.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Throughput::usage = "\<Throughput[arch][param] the number of samples \
processed per clock cycle on average. Note that the second parameter already \
indicates the number of simultaneous inputs, so the throughput is effectively \
the second parameter divided by the response time.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ResponseTime::usage\  = \ "\<ResponseTime[arch][param] the number of \
clock cycles between accepting inputs.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Usage::usage = "\<Usage[arch][param] the fraction of the hardware \
that is busy at any given moment.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ConstantsNr::usage = "\<ConstantsNr[arch][param] the number of \
constants in the architecture. This gives an indication of how many locations \
have to be updated if the filter coefficients change.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ConstantsUniqueNr::usage = "\<ConstantsUniqueNr[arch][param] the \
number of unique constants in the architecture. This gives an indication of \
how many calculations have to be performed if the filter coefficients change. \
Note that this is a worst case number, in special cases more constants may be \
equal.\>";\)\)], "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Function Descriptions", "Section",
  InitializationCell->True],

Cell["\<\
Note that these function descriptions only contain default values. They \
should be overridden by architectures that do not adhere to these defaults.\
\>", "Text",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Begin["\<`Private`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`Main`Private`"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(ParamNr[arch_] := 2\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamDescription[arch_]\)[1] := "\<Order of the filter.\>"\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(ParamDescription[arch_]\)[
        2] := "\<Number of simultaneous inputs.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamMin[arch_]\)[param_, n_] := 1\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamMax[arch_]\)[param_, n_] := Infinity\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamFindHardwareMultipliers[arch_]\)[n_, h_] := 
      Module[{sol}, \[IndentingNewLine]Off[Solve::"\<ifun\>"]; 
        sol\  = \ 
          Solve[\(HardwareMultipliers[arch]\)[
                Table[p[i], {i, ParamNr[arch]}] /. \ {p[1] \[Rule] \ 
                      n}]\  \[Equal] \ h, 
            Table[p[i], {i, 2, ParamNr[arch]}]]; 
        If[Length[sol] \[Equal] 0, \ 
          Table[0, {i, 
              ParamNr[arch]}], \(Table[p[i], {i, ParamNr[arch]}]\  /. \ 
              sol[\([1]\)]\)\  /. \ {p[1] \[Rule] \ n}]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamFindThroughput[arch_]\)[n_, t_] := 
      Module[{sol}, \[IndentingNewLine]Off[Solve::"\<ifun\>"]; 
        sol\  = \ 
          Solve[\(Throughput[arch]\)[
                Table[p[i], {i, ParamNr[arch]}] /. \ {p[1] \[Rule] \ 
                      n}]\  \[Equal] \ t, 
            Table[p[i], {i, 2, ParamNr[arch]}]]; 
        If[Length[sol] \[Equal] 0, \ 
          Table[0, {i, 
              ParamNr[arch]}], \(Table[p[i], {i, ParamNr[arch]}]\  /. \ 
              sol[\([1]\)]\)\  /. \ {p[1] \[Rule] \ n}]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamTest[arch_]\)[param_, n_] := 
      And[param[\([n]\)] \[GreaterEqual] \ \(ParamMin[arch]\)[param, n], \ 
        param[\([n]\)] \[LessEqual] \ \(ParamMax[arch]\)[param, 
            n]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamTest[arch_]\)[param_] := 
      And @@ Table[\(ParamTest[arch]\)[param, n], {n, 
            Length[param]}]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ClockCycle[arch_]\)[
        param_] := \(WorkUnit[arch]\)[param]/\(NeededCycles[arch]\)[
          param, \(WorkUnit[arch]\)[param]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(NeededCycles[arch_]\)[param_, {mults_, adds_}] := 1\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Throughput[arch_]\)[param_] := 
      param[\([2]\)]\ /\ \(ResponseTime[arch]\)[param]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ResponseTime[arch_]\)[param_] := \ \(NeededCycles[arch]\)[
        param, \(WorkUnit[arch]\)[param]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Usage[arch_]\)[param_] := 
      1/\(NeededCycles[arch]\)[param, \(WorkUnit[arch]\)[param]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ConstantsNr[arch_]\)[param_] := \(HardwareMultipliers[arch]\)[
        param]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ConstantsUniqueNr[arch_]\)[param_] := \(ConstantsNr[arch]\)[
        param]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(End[]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`Main`Private`"\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Loading the Architectures", "Section",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Get["\<Architecture`\>" <> #1\  <> "\<`\>"] &, 
      ArchitectureList]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \({Null, Null, Null, Null}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(EndPackage[]\)], "Input",
  InitializationCell->True]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1400}, {0, 975}},
AutoGeneratedPackage->Automatic,
WindowSize->{496, 542},
WindowMargins->{{18, Automatic}, {Automatic, 23}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 80, 1, 150, "Title",
  InitializationCell->True],
Cell[1859, 56, 57, 1, 51, "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1941, 61, 99, 2, 30, "Input",
  InitializationCell->True],
Cell[2043, 65, 54, 1, 29, "Output"]
}, Open  ]],
Cell[2112, 69, 207, 4, 70, "Input",
  InitializationCell->True],
Cell[2322, 75, 178, 4, 70, "Input",
  InitializationCell->True],
Cell[2503, 81, 146, 3, 50, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[2674, 88, 65, 1, 73, "Section",
  InitializationCell->True],
Cell[2742, 91, 160, 3, 70, "Input",
  InitializationCell->True],
Cell[2905, 96, 196, 3, 90, "Input",
  InitializationCell->True],
Cell[3104, 101, 236, 4, 90, "Input",
  InitializationCell->True],
Cell[3343, 107, 187, 3, 90, "Input",
  InitializationCell->True],
Cell[3533, 112, 207, 4, 90, "Input",
  InitializationCell->True],
Cell[3743, 118, 236, 4, 90, "Input",
  InitializationCell->True],
Cell[3982, 124, 238, 4, 90, "Input",
  InitializationCell->True],
Cell[4223, 130, 238, 4, 90, "Input",
  InitializationCell->True],
Cell[4464, 136, 435, 7, 190, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[4924, 147, 437, 7, 190, "Input",
  InitializationCell->True],
Cell[5364, 156, 375, 5, 143, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5776, 166, 324, 6, 150, "Input",
  InitializationCell->True],
Cell[6103, 174, 274, 4, 105, "Output"]
}, Open  ]],
Cell[6392, 181, 356, 6, 150, "Input",
  InitializationCell->True],
Cell[6751, 189, 364, 6, 170, "Input",
  InitializationCell->True],
Cell[7118, 197, 385, 6, 170, "Input",
  InitializationCell->True],
Cell[7506, 205, 197, 4, 90, "Input",
  InitializationCell->True],
Cell[7706, 211, 180, 3, 70, "Input",
  InitializationCell->True],
Cell[7889, 216, 183, 3, 70, "Input",
  InitializationCell->True],
Cell[8075, 221, 170, 3, 70, "Input",
  InitializationCell->True],
Cell[8248, 226, 349, 5, 150, "Input",
  InitializationCell->True],
Cell[8600, 233, 178, 3, 70, "Input",
  InitializationCell->True],
Cell[8781, 238, 170, 3, 70, "Input",
  InitializationCell->True],
Cell[8954, 243, 267, 4, 130, "Input",
  InitializationCell->True],
Cell[9224, 249, 380, 6, 170, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[9641, 260, 68, 1, 43, "Section",
  InitializationCell->True],
Cell[9712, 263, 201, 4, 52, "Text",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[9938, 271, 83, 2, 30, "Input",
  InitializationCell->True],
Cell[10024, 275, 62, 1, 29, "Output"]
}, Open  ]],
Cell[10101, 279, 80, 2, 30, "Input",
  InitializationCell->True],
Cell[10184, 283, 123, 3, 30, "Input",
  InitializationCell->True],
Cell[10310, 288, 140, 3, 50, "Input",
  InitializationCell->True],
Cell[10453, 293, 97, 2, 30, "Input",
  InitializationCell->True],
Cell[10553, 297, 104, 2, 30, "Input",
  InitializationCell->True],
Cell[10660, 301, 597, 12, 230, "Input",
  InitializationCell->True],
Cell[11260, 315, 579, 12, 230, "Input",
  InitializationCell->True],
Cell[11842, 329, 254, 5, 70, "Input",
  InitializationCell->True],
Cell[12099, 336, 176, 4, 70, "Input",
  InitializationCell->True],
Cell[12278, 342, 196, 4, 70, "Input",
  InitializationCell->True],
Cell[12477, 348, 114, 2, 50, "Input",
  InitializationCell->True],
Cell[12594, 352, 149, 3, 50, "Input",
  InitializationCell->True],
Cell[12746, 357, 163, 3, 50, "Input",
  InitializationCell->True],
Cell[12912, 362, 154, 3, 50, "Input",
  InitializationCell->True],
Cell[13069, 367, 140, 3, 50, "Input",
  InitializationCell->True],
Cell[13212, 372, 138, 3, 50, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[13375, 379, 66, 2, 30, "Input",
  InitializationCell->True],
Cell[13444, 383, 62, 1, 29, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[13555, 390, 72, 1, 43, "Section",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[13652, 395, 136, 3, 50, "Input",
  InitializationCell->True],
Cell[13791, 400, 58, 1, 70, "Output"]
}, Open  ]],
Cell[13864, 404, 73, 2, 30, "Input",
  InitializationCell->True]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

