(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      4321,        138]*)
(*NotebookOutlinePosition[      5000,        161]*)
(*  CellTagsIndexPosition[      4956,        157]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["The IIR Filter Architecture Package", "Title",
  InitializationCell->True],

Cell["Scattered Look-Ahead", "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(BeginPackage["\<Architecture`SLA`\>", "\<Architecture`Main`\>"]\)], \
"Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`SLA`"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Begin["\<`Private`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`SLA`Private`"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Description["\<SLA\>"] := "\<The SLA architecture is based on the \
scattered look-ahead approach with decomposition. We assumed direct form \
implementations running in parallel for the recursive part and  parallel fir \
filters for the nonrecursive part. state space form of the filter. Because of \
the decomposition this architecture has the restriction that the second \
parameter needs to be a power of two. Note that the characteristics of this \
architecture may change if a different implementation of the nonrecursive and \
recursive parts was chosen (Fast FIR filtering techniques for the \
nonrecursive part for example). See: zhou88twolevel, brent91stabilized, \
parhi88pipelined, parhi89pipeline.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamTest["\<SLA\>"]\)[param_, 2] := 
      And[param[\([2]\)] \[GreaterEqual] \ 1, 
        Round[Log[2, param[\([2]\)]]] \[Equal] \ 
          Log[2, param[\([2]\)]]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(WorkUnit["\<SLA\>"]\)[param_] := {1, param[\([1]\)]}\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareMultipliers["\<SLA\>"]\)[{N_, P_}] := 
      P\ \((Log[2, P]\  + \ 2)\)\ N\  + \ P\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareAdders["\<SLA\>"]\)[{N_, P_}] := 
      N\ P\ \((Log[2, P]\  + \ 2)\)\  + \ P\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareBuffers["\<SLA\>"]\)[{N_, P_}] := \ P\ N\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Latency["\<SLA\>"]\)[{N_, P_}] := \(ResponseTime[arch_]\)[
          param_]\ *\ \((\((Log[2, P] + 1)\) N\  + \ 2)\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(End[]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`SLA`Private`"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(EndPackage[]\)], "Input",
  InitializationCell->True]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1400}, {0, 975}},
AutoGeneratedPackage->Automatic,
WindowSize->{496, 740},
WindowMargins->{{Automatic, 238}, {Automatic, 24}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 80, 1, 150, "Title",
  InitializationCell->True],
Cell[1859, 56, 68, 1, 51, "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1952, 61, 126, 3, 50, "Input",
  InitializationCell->True],
Cell[2081, 66, 53, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2171, 72, 83, 2, 30, "Input",
  InitializationCell->True],
Cell[2257, 76, 61, 1, 29, "Output"]
}, Open  ]],
Cell[2333, 80, 774, 11, 350, "Input",
  InitializationCell->True],
Cell[3110, 93, 231, 5, 70, "Input",
  InitializationCell->True],
Cell[3344, 100, 115, 2, 30, "Input",
  InitializationCell->True],
Cell[3462, 104, 153, 3, 50, "Input",
  InitializationCell->True],
Cell[3618, 109, 148, 3, 30, "Input",
  InitializationCell->True],
Cell[3769, 114, 111, 2, 30, "Input",
  InitializationCell->True],
Cell[3883, 118, 179, 3, 50, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[4087, 125, 66, 2, 30, "Input",
  InitializationCell->True],
Cell[4156, 129, 61, 1, 29, "Output"]
}, Open  ]],
Cell[4232, 133, 73, 2, 30, "Input",
  InitializationCell->True]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

