(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      5168,        151]*)
(*NotebookOutlinePosition[      5847,        174]*)
(*  CellTagsIndexPosition[      5803,        170]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["The IIR Filter Architecture Package", "Title",
  InitializationCell->True],

Cell["Scattered Look-Ahead with incremental output computation", "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(BeginPackage["\<Architecture`SLAIncr`\>", "\<Architecture`Main`\>"]\)], \
"Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`SLAIncr`"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Begin["\<`Private`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`SLAIncr`Private`"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Description["\<SLAIncr\>"] := "\<The SLA architecture with incremental \
output computation is based on the scattered look-ahead approach with \
decomposition. We assumed direct form implementations running in parallel for \
the recursive part and  parallel fir filters for the nonrecursive part of the \
first N outputs. The other outputs are calculated using incremental output \
computation. Because of the decomposition this architecture has the \
restriction that the second parameter needs to be a power of two. Note that \
the characteristics of this architecture may change if a different \
implementation of the nonrecursive and recursive parts was chosen (Fast FIR \
filtering techniques for the nonrecursive part for example). Note that the \
incremental output computation is not very effective; The filter consists of \
an IIR stage and several FIR stages. Only one of these stages (the last one) \
can be reduced, because the others all have to all their outputs since they \
are needed by the next stage to calculate their outputs. See: zhou88twolevel, \
brent91stabilized, parhi88pipelined, parhi89pipeline.\>"\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ParamTest["\<SLAIncr\>"]\)[param_, 2] := 
      And[param[\([2]\)] \[GreaterEqual] \ 1, 
        Round[Log[2, param[\([2]\)]]] \[Equal] \ 
          Log[2, param[\([2]\)]]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(WorkUnit["\<SLAIncr\>"]\)[param_] := {1, param[\([1]\)]}\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareMultipliers["\<SLAIncr\>"]\)[{N_, P_}] := 
      If[P \[GreaterEqual] \ N, 
        P\ \((Log[2, P]\  + \ 2)\)\ N\  + \ 
          P\  - \((P - N)\) N\  + \ \((P - N)\) \((2  N + 1)\), 
        P\ \((Log[2, P]\  + \ 2)\)\ N\  + \ P]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareAdders["\<SLAIncr\>"]\)[{N_, P_}] := 
      If[P \[GreaterEqual] \ N, 
        P\ \((Log[2, P]\  + \ 2)\)\ N\  + \ 
          P\  - \((P - N)\) N\  + \ \((P - N)\) \((2  N + 1)\), 
        P\ \((Log[2, P]\  + \ 2)\)\ N\  + \ P]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(HardwareBuffers["\<SLAIncr\>"]\)[{N_, P_}] := \ P\ N\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Latency["\<SLAIncr\>"]\)[{N_, 
          P_}] := \(ResponseTime[arch_]\)[
            param_]\ *\ \((\((Log[2, P] + 1)\) N\  + \ 2)\)\  + \ 
        If[P \[GreaterEqual] \ N, \ P - N, 0]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(End[]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("Architecture`SLAIncr`Private`"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(EndPackage[]\)], "Input",
  InitializationCell->True]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1400}, {0, 975}},
AutoGeneratedPackage->Automatic,
WindowSize->{496, 740},
WindowMargins->{{140, Automatic}, {Automatic, 17}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 80, 1, 150, "Title",
  InitializationCell->True],
Cell[1859, 56, 104, 1, 80, "Subtitle",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1988, 61, 130, 3, 50, "Input",
  InitializationCell->True],
Cell[2121, 66, 57, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2215, 72, 83, 2, 30, "Input",
  InitializationCell->True],
Cell[2301, 76, 65, 1, 29, "Output"]
}, Open  ]],
Cell[2381, 80, 1187, 16, 530, "Input",
  InitializationCell->True],
Cell[3571, 98, 235, 5, 70, "Input",
  InitializationCell->True],
Cell[3809, 105, 119, 2, 30, "Input",
  InitializationCell->True],
Cell[3931, 109, 303, 6, 70, "Input",
  InitializationCell->True],
Cell[4237, 117, 298, 6, 70, "Input",
  InitializationCell->True],
Cell[4538, 125, 115, 2, 30, "Input",
  InitializationCell->True],
Cell[4656, 129, 249, 5, 70, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[4930, 138, 66, 2, 30, "Input",
  InitializationCell->True],
Cell[4999, 142, 65, 1, 29, "Output"]
}, Open  ]],
Cell[5079, 146, 73, 2, 30, "Input",
  InitializationCell->True]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

