(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      9926,        272]*)
(*NotebookOutlinePosition[     10573,        294]*)
(*  CellTagsIndexPosition[     10529,        290]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Hardware efficiencies of the Architectures with respect to IIR filters\
\>", "Title"],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[BoxData[
    \(<< Graphics`MultipleListPlot`\)], "Input"],

Cell[BoxData[
    \(<< \ Architecture`Main`\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ArchList\  = \ ArchitectureListIIR\)], "Input"],

Cell[BoxData[
    \({"BlockState", "IncrementalBlockState", "ParallelBlockState", "SLA", 
      "SLAIncr", "CLAIncr"}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(RangeMax = 128;\)\)], "Input"],

Cell[BoxData[
    \(\(GetParam[arch_]\)[N_, K_] := \(ParamFindThroughput[arch]\)[N, 
        K]\)], "Input"],

Cell[BoxData[
    \(\(Efficiency[arch_]\)[N_, K_] := 
      If[\(\(GetParam[arch]\)[N, K]\)[\([1]\)] \[NotEqual] \ 
          0, \((2  N\  + \ 1)\) 
          K\ /\ \(HardwareMultipliers[arch]\)[\(GetParam[arch]\)[N, K]], 
        0]\)], "Input"],

Cell[BoxData[
    \(EffPlot3D[arch_] := 
      Plot3D[\(Efficiency[arch]\)[n, K], {K, 1, RangeMax}, {n, 1, 
          RangeMax}, \ AxesLabel \[Rule] \ {"\<K\>", "\<N\>", "\<\>"}, 
        PlotRange \[Rule] {0, 1}, \ PlotLabel \[Rule] \ arch]\)], "Input"],

Cell[BoxData[
    \(\(EfficiencyList[N_]\)[arch_] := 
      Select[Table[
          If[\(ParamTest[arch]\)[\(GetParam[arch]\)[N, 
                K]], {K, \(Efficiency[arch]\)[N, K]}, ], {K, 1, RangeMax, 
            1}], \((Length[#1] \[Equal] 2)\) &]\)], "Input"],

Cell[BoxData[
    \(\(EfficiencyPlotList[N_]\)[arch_] := 
      If[\(EfficiencyList[N]\)[arch] \[NotEqual] \ {}, \(EfficiencyList[N]\)[
          arch], Table[{K, 0}, {K, 1, RangeMax, 1}]]\)], "Input"],

Cell[BoxData[
    \(PlotSeries[N_] := 
      MultipleListPlot[
        Map[EfficiencyPlotList[N], \ 
          ArchList], \[IndentingNewLine]PlotLegend \[Rule] 
          ArchList, \[IndentingNewLine]PlotJoined \[Rule] 
          True, \[IndentingNewLine]PlotStyle \[Rule] 
          Table[Hue[x/Length[ArchList]], {x, 1, 
              Length[ArchList]}], \[IndentingNewLine]SymbolStyle \[Rule] 
          Table[Hue[x/Length[ArchList]], {x, 1, 
              Length[ArchList]}], \[IndentingNewLine]PlotLabel -> \
"\<Efficiency for N = \>" <> ToString[N], \ 
        PlotRange \[Rule] {0, 
            1}, \[IndentingNewLine]AxesLabel \[Rule] {"\<K\>", \
"\<Efficiency\>"}]\)], "Input"],

Cell[BoxData[
    \(\(EfficiencyList2[K_]\)[arch_] := 
      Select[Table[
          If[\(ParamTest[arch]\)[\(GetParam[arch]\)[N, 
                K]], {N, \(Efficiency[arch]\)[N, K]}, ], {N, 1, RangeMax, 
            1}], \((Length[#1] \[Equal] 2)\) &]\)], "Input"],

Cell[BoxData[
    \(\(EfficiencyPlotList2[K_]\)[arch_] := 
      If[\(EfficiencyList2[K]\)[arch] \[NotEqual] \ {}, \(EfficiencyList2[
            K]\)[arch], Table[{N, 0}, {N, 1, RangeMax, 1}]]\)], "Input"],

Cell[BoxData[
    \(PlotSeries2[K_] := 
      MultipleListPlot[
        Map[EfficiencyPlotList2[K], \ 
          ArchList], \[IndentingNewLine]PlotLegend \[Rule] 
          ArchList, \[IndentingNewLine]PlotJoined \[Rule] 
          True, \[IndentingNewLine]PlotStyle \[Rule] 
          Table[Hue[x/Length[ArchList]], {x, 1, 
              Length[ArchList]}], \[IndentingNewLine]SymbolStyle \[Rule] 
          Table[Hue[x/Length[ArchList]], {x, 1, 
              Length[ArchList]}], \[IndentingNewLine]PlotLabel -> \
"\<Efficiency for K = \>" <> ToString[K], \ 
        PlotRange \[Rule] {0, 
            1}, \[IndentingNewLine]AxesLabel \[Rule] {"\<N\>", \
"\<Efficiency\>"}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Experimentation", "Section"],

Cell["Uncomment to experiment with other hardware possibilities", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \("\<NeededCycles[arch_][param_,{m_,a_}]:=Max[2 m,1]\>"\)], "Input"],

Cell[BoxData[
    \("NeededCycles[arch_][param_,{m_,a_}]:=Max[2 m,1]"\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Architecture Descriptions", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Map[Print[#1 <> "\<\n\>"\  <> \ Description[#1]] &, 
        ArchList];\)\)], "Input"],

Cell[BoxData[
    \("BlockState\nThe block-state architecture is based on the state space \
form of the filter. For the performance calculations we assumed that the \
decomposition of the filter into parallel second order sections is known and \
that there are no first order sections. See: lu85fast, parhi87block, \
parhi89pipeline2."\)], "Print"],

Cell[BoxData[
    \("IncrementalBlockState\nThe incremental block-state architecture, like \
the block-state architecture, is based on the state space form of the filter. \
For the performance calculations we assumed that the decomposition of the \
filter into parallel second order sections is known and that there are no \
first order sections. The third parameter is the increment size, but it can \
be omitted. In this case the third parameter will be chosen such that the \
number of multipliers in the implementation is minimal.See: parhi87block, \
parhi89pipeline2."\)], "Print"],

Cell[BoxData[
    \("ParallelBlockState\nThe parallel block-state architecture is, like the \
block-state, based on the state space form of the filter. For the performance \
calculations we assumed that the decomposition of the filter into parallel \
second order sections is known and that there are no first order sections. \
See: nikias84fast, parhi87block."\)], "Print"],

Cell[BoxData[
    \("SLA\nThe SLA architecture is based on the scattered look-ahead \
approach with decomposition. We assumed direct form implementations running \
in parallel for the recursive part and  parallel fir filters for the \
nonrecursive part. state space form of the filter. Because of the \
decomposition this architecture has the restriction that the second parameter \
needs to be a power of two. Note that the characteristics of this \
architecture may change if a different implementation of the nonrecursive and \
recursive parts was chosen (Fast FIR filtering techniques for the \
nonrecursive part for example). See: zhou88twolevel, brent91stabilized, \
parhi88pipelined, parhi89pipeline."\)], "Print"],

Cell[BoxData[
    \("SLAIncr\nThe SLA architecture with incremental output computation is \
based on the scattered look-ahead approach with decomposition. We assumed \
direct form implementations running in parallel for the recursive part and  \
parallel fir filters for the nonrecursive part of the first N outputs. The \
other outputs are calculated using incremental output computation. Because of \
the decomposition this architecture has the restriction that the second \
parameter needs to be a power of two. Note that the characteristics of this \
architecture may change if a different implementation of the nonrecursive and \
recursive parts was chosen (Fast FIR filtering techniques for the \
nonrecursive part for example). Note that the incremental output computation \
is not very effective; The filter consists of an IIR stage and several FIR \
stages. Only one of these stages (the last one) can be reduced, because the \
others all have to all their outputs since they are needed by the next stage \
to calculate their outputs. See: zhou88twolevel, brent91stabilized, \
parhi88pipelined, parhi89pipeline."\)], "Print"],

Cell[BoxData[
    \("CLAIncr\nThe CLAIncr architecture is based on the clustered look-ahead \
approach with incremental output computation. Also called Direct Form Block \
Filter by Parhi. Fast FIR filter techniques might be used to improve this \
architecture further. Note that this filter is not necessarily stable. See: \
parhi89pipeline2."\)], "Print"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Individual Efficiencies", "Section"],

Cell[BoxData[
    \(Map[EffPlot3D, ArchList]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Efficiencies Compared", "Section"],

Cell[CellGroupData[{

Cell["Fixed N", "Subsection"],

Cell[BoxData[
    \(Map[PlotSeries, {8, 16, 32, 64}]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fixed K", "Subsection"],

Cell[BoxData[
    \(Map[PlotSeries2, {8, 16, 32, 64}]\)], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1400}, {0, 975}},
WindowSize->{783, 640},
WindowMargins->{{112, Automatic}, {Automatic, -15}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 95, 2, 150, "Title"],

Cell[CellGroupData[{
Cell[1896, 59, 33, 0, 73, "Section"],
Cell[1932, 61, 62, 1, 30, "Input"],
Cell[1997, 64, 56, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2078, 69, 67, 1, 30, "Input"],
Cell[2148, 72, 131, 2, 70, "Output"]
}, Open  ]],
Cell[2294, 77, 52, 1, 30, "Input"],
Cell[2349, 80, 108, 2, 30, "Input"],
Cell[2460, 84, 246, 5, 70, "Input"],
Cell[2709, 91, 254, 4, 50, "Input"],
Cell[2966, 97, 265, 5, 70, "Input"],
Cell[3234, 104, 201, 3, 50, "Input"],
Cell[3438, 109, 686, 14, 150, "Input"],
Cell[4127, 125, 266, 5, 70, "Input"],
Cell[4396, 132, 206, 3, 50, "Input"],
Cell[4605, 137, 688, 14, 150, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[5330, 156, 34, 0, 43, "Section"],
Cell[5367, 158, 73, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[5465, 162, 86, 1, 30, "Input"],
Cell[5554, 165, 83, 1, 70, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[5686, 172, 44, 0, 43, "Section"],

Cell[CellGroupData[{
Cell[5755, 176, 108, 2, 30, "Input"],
Cell[5866, 180, 348, 5, 70, "Print"],
Cell[6217, 187, 586, 8, 70, "Print"],
Cell[6806, 197, 374, 5, 70, "Print"],
Cell[7183, 204, 721, 10, 70, "Print"],
Cell[7907, 216, 1134, 15, 70, "Print"],
Cell[9044, 233, 357, 5, 70, "Print"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[9450, 244, 42, 0, 43, "Section"],
Cell[9495, 246, 57, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9589, 252, 40, 0, 73, "Section"],

Cell[CellGroupData[{
Cell[9654, 256, 29, 0, 38, "Subsection"],
Cell[9686, 258, 65, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9788, 264, 29, 0, 38, "Subsection"],
Cell[9820, 266, 66, 1, 30, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

